@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2022
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -116,6 +116,24 @@ class BackupInfoBadInitialisation(BackupException):
     """
 
 
+class BackupPreconditionException(BackupException):
+    """
+    Exception for a backup precondition not being met
+    """
+
+
+class SnapshotBackupException(BackupException):
+    """
+    Exception for snapshot backups
+    """
+
+
+class SnapshotInstanceNotFoundException(SnapshotBackupException):
+    """
+    Raised when the VM instance related to a snapshot backup cannot be found
+    """
+
+
 class SyncError(SyncException):
     """
     Synchronisation error
@@ -188,6 +206,24 @@ class CompressionIncompatibility(CompressionException):
     """
 
 
+class UnsupportedCompressionFormat(CompressionException):
+    """
+    Exception for unsupported compression format
+    """
+
+
+class FileNotFoundException(CompressionException):
+    """
+    Exception for file not found in archive
+    """
+
+
+class CommandNotFoundException(CommandException):
+    """
+    Exception for command not found
+    """
+
+
 class FsOperationFailed(CommandException):
     """
     Exception which represents a failed execution of a command on FS
@@ -233,6 +269,12 @@ class PostgresConnectionError(PostgresException):
             return ""
 
 
+class PostgresConnectionLost(PostgresException):
+    """
+    The Postgres connection was lost during an execution
+    """
+
+
 class PostgresAppNameError(PostgresConnectionError):
     """
     Error setting application name with PostgreSQL server
@@ -251,6 +293,12 @@ class BackupFunctionsAccessRequired(PostgresException):
     """
 
 
+class PostgresCheckpointPrivilegesRequired(PostgresException):
+    """
+    Superuser or role 'pg_checkpoint' is required
+    """
+
+
 class PostgresIsInRecovery(PostgresException):
     """
     PostgreSQL is in recovery, so no write operations are allowed
@@ -263,6 +311,13 @@ class PostgresUnsupportedFeature(PostgresException):
     """
 
 
+class PostgresObsoleteFeature(PostgresException):
+    """
+    Obsolete feature, i.e. one which has been deprecated and since
+    removed.
+    """
+
+
 class PostgresDuplicateReplicationSlot(PostgresException):
     """
     The creation of a physical replication slot failed because
@@ -310,6 +365,12 @@ class BadXlogSegmentName(WALFileException):
     """
 
 
+class BadXlogPrefix(WALFileException):
+    """
+    Exception for a bad xlog prefix
+    """
+
+
 class BadHistoryFileContents(WALFileException):
     """
     Exception for a corrupted history file
@@ -345,6 +406,12 @@ class RecoveryException(BarmanException):
     """
 
 
+class RecoveryPreconditionException(RecoveryException):
+    """
+    Exception for a recovery precondition not being met
+    """
+
+
 class RecoveryTargetActionException(RecoveryException):
     """
     Exception for a wrong recovery target action
@@ -385,3 +452,15 @@ class InvalidRetentionPolicy(BarmanException):
     """
     Exception raised when a retention policy cannot be parsed.
     """
+
+
+class BackupManifestException(BarmanException):
+    """
+    Exception raised when there is a problem with the backup manifest.
+    """
+
+
+class EncryptionCommandException(CommandFailedException):
+    """
+    Exception representing a failed encryption command.
+    """
