@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# © Copyright EnterpriseDB UK Limited 2011-2023
+# © Copyright EnterpriseDB UK Limited 2011-2025
 #
 # This file is part of Barman.
 #
@@ -23,23 +23,26 @@ This module represents the interface towards a PostgreSQL server.
 import atexit
 import datetime
 import logging
+import os
+import signal
+import threading
+import time
 from abc import ABCMeta
 from multiprocessing import Process, Queue
 
-try:
-    from queue import Empty
-except ImportError:
-    from Queue import Empty
-
 import psycopg2
 from psycopg2.errorcodes import DUPLICATE_OBJECT, OBJECT_IN_USE, UNDEFINED_OBJECT
 from psycopg2.extensions import STATUS_IN_TRANSACTION, STATUS_READY
 from psycopg2.extras import DictCursor, NamedTupleCursor
 
+from barman import xlog
 from barman.exceptions import (
+    BackupFunctionsAccessRequired,
     ConninfoException,
     PostgresAppNameError,
+    PostgresCheckpointPrivilegesRequired,
     PostgresConnectionError,
+    PostgresConnectionLost,
     PostgresDuplicateReplicationSlot,
     PostgresException,
     PostgresInvalidReplicationSlot,
@@ -47,8 +50,6 @@ from barman.exceptions import (
     PostgresObsoleteFeature,
     PostgresReplicationSlotInUse,
     PostgresReplicationSlotsFull,
-    BackupFunctionsAccessRequired,
-    PostgresCheckpointPrivilegesRequired,
     PostgresUnsupportedFeature,
 )
 from barman.infofile import Tablespace
@@ -56,6 +57,11 @@ from barman.postgres_plumbing import function_name_map
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import force_str, simplify_version, with_metaclass
 
+try:
+    from queue import Empty
+except ImportError:
+    from Queue import Empty
+
 # This is necessary because the CONFIGURATION_LIMIT_EXCEEDED constant
 # has been added in psycopg2 2.5, but Barman supports version 2.4.2+ so
 # in case of import error we declare a constant providing the correct value.
@@ -457,6 +463,8 @@ class PostgreSQLConnection(PostgreSQL):
     WALSTREAMER = 2
     ANY_STREAMING_CLIENT = (STANDBY, WALSTREAMER)
 
+    HEARTBEAT_QUERY = "SELECT 1"
+
     def __init__(
         self,
         conninfo,
@@ -498,6 +506,11 @@ class PostgreSQLConnection(PostgreSQL):
                 raise PostgresAppNameError(force_str(e).strip())
         return self._conn
 
+    @property
+    def has_connection(self):
+        """Checks if the Postgres connection has already been set"""
+        return True if self._conn is not None else False
+
     @property
     def server_txt_version(self):
         """
@@ -598,11 +611,11 @@ class PostgreSQLConnection(PostgreSQL):
           OR
           (
             (
-              pg_has_role(CURRENT_USER, 'pg_monitor', 'MEMBER')
+              pg_has_role(CURRENT_USER, 'pg_monitor', 'USAGE')
               OR
               (
-                pg_has_role(CURRENT_USER, 'pg_read_all_settings', 'MEMBER')
-                AND pg_has_role(CURRENT_USER, 'pg_read_all_stats', 'MEMBER')
+                pg_has_role(CURRENT_USER, 'pg_read_all_settings', 'USAGE')
+                AND pg_has_role(CURRENT_USER, 'pg_read_all_stats', 'USAGE')
               )
             )
             AND
@@ -652,7 +665,7 @@ class PostgreSQLConnection(PostgreSQL):
             return True
         else:
             role_check_query = (
-                "select pg_has_role(CURRENT_USER ,'pg_checkpoint', 'MEMBER');"
+                "select pg_has_role(CURRENT_USER ,'pg_checkpoint', 'USAGE');"
             )
             try:
                 cur = self._cursor()
@@ -665,6 +678,42 @@ class PostgreSQLConnection(PostgreSQL):
                 )
                 return None
 
+    @property
+    def has_monitoring_privileges(self):
+        """
+        Check whether the current user can access monitoring information.
+
+        Returns ``True`` if the current user is a superuser or if the user has the
+        necessary privileges to monitor system status.
+
+        :rtype: bool
+        :return: ``True`` if the current user can access monitoring information.
+        """
+        if self.is_superuser:
+            return True
+        else:
+            monitoring_check_query = """
+            SELECT
+            (
+                pg_has_role(CURRENT_USER, 'pg_monitor', 'USAGE')
+                OR
+                (
+                    pg_has_role(CURRENT_USER, 'pg_read_all_settings', 'USAGE')
+                    AND pg_has_role(CURRENT_USER, 'pg_read_all_stats', 'USAGE')
+                )
+            )
+            """
+            try:
+                cur = self._cursor()
+                cur.execute(monitoring_check_query)
+                return cur.fetchone()[0]
+            except (PostgresConnectionError, psycopg2.Error) as e:
+                _logger.debug(
+                    "Error checking privileges for functions needed for monitoring: %s",
+                    force_str(e).strip(),
+                )
+                return None
+
     @property
     def current_xlog_info(self):
         """
@@ -691,7 +740,20 @@ class PostgreSQLConnection(PostgreSQL):
                     "CURRENT_TIMESTAMP AS timestamp "
                     "FROM {pg_current_wal_lsn}() AS location".format(**self.name_map)
                 )
-                return cur.fetchone()
+                result = cur.fetchone()
+                # The following change was made in PostgreSQL 17:
+                # The functions pg_walfile_name() and pg_walfile_name_offset() used to
+                # report the previous LSN segment number when the LSN was on a file
+                # segment boundary; it now returns the current LSN segment.
+                # For PostgreSQL 17 and above, when the WAL file offset is 0 (i.e., at
+                # the start of a new segment), use the name of the previous WAL segment
+                # to maintain compatibility and consistent behavior across PostgreSQL
+                # major versions.
+                if self.server_version >= 170000 and result["file_offset"] == 0:
+                    result["file_name"] = xlog.previous_segment_name(
+                        result["file_name"], self.xlog_segment_size
+                    )
+                return result
             else:
                 cur.execute(
                     "SELECT location, "
@@ -938,6 +1000,7 @@ class PostgreSQLConnection(PostgreSQL):
 
             result["is_superuser"] = self.is_superuser
             result["has_backup_privileges"] = self.has_backup_privileges
+            result["has_monitoring_privileges"] = self.has_monitoring_privileges
             result["is_in_recovery"] = self.is_in_recovery
             result["server_txt_version"] = self.server_txt_version
             result["version_supported"] = self.is_minimal_postgres_version()
@@ -1344,11 +1407,18 @@ class PostgreSQLConnection(PostgreSQL):
 
             cur = conn.cursor()
             # Collect the xlog file name before the switch
-            cur.execute(
-                "SELECT {pg_walfile_name}("
-                "{pg_current_wal_insert_lsn}())".format(**self.name_map)
+            query = "SELECT {pg_walfile_name}({pg_current_wal_insert_lsn}())".format(
+                **self.name_map
             )
-            pre_switch = cur.fetchone()[0]
+            if self.server_version >= 170000:
+                query = (
+                    "SELECT ({pg_walfile_name_offset}(location)).* "
+                    "FROM {pg_current_wal_insert_lsn}() AS location"
+                ).format(**self.name_map)
+            cur.execute(query)
+            pre_switch_result = cur.fetchone()
+            pre_switch = pre_switch_result[0]
+
             # Switch
             cur.execute(
                 "SELECT {pg_walfile_name}({pg_switch_wal}())".format(**self.name_map)
@@ -1359,7 +1429,20 @@ class PostgreSQLConnection(PostgreSQL):
                 "{pg_current_wal_insert_lsn}())".format(**self.name_map)
             )
             post_switch = cur.fetchone()[0]
+
             if pre_switch < post_switch:
+                # The following change was made in PostgreSQL 17:
+                # The functions pg_walfile_name() and pg_walfile_name_offset() used to
+                # report the previous LSN segment number when the LSN was on a file
+                # segment boundary; it now returns the current LSN segment.
+                # For PostgreSQL 17 and above, when the WAL file offset is 0 (i.e., at
+                # the start of a new segment), use the name of the previous WAL segment
+                # to maintain compatibility and consistent behavior across PostgreSQL
+                # major versions.
+                if self.server_version >= 170000 and pre_switch_result[1] == 0:
+                    pre_switch = xlog.previous_segment_name(
+                        pre_switch_result[0], self.xlog_segment_size
+                    )
                 return pre_switch
             else:
                 return ""
@@ -1393,7 +1476,7 @@ class PostgreSQLConnection(PostgreSQL):
         try:
             cur = self._cursor(cursor_factory=NamedTupleCursor)
 
-            if not self.has_backup_privileges:
+            if not self.has_monitoring_privileges:
                 raise BackupFunctionsAccessRequired(
                     "Postgres user '%s' is missing required privileges "
                     '(see "Preliminary steps" in the Barman manual)'
@@ -1565,6 +1648,26 @@ class PostgreSQLConnection(PostgreSQL):
             # the format of the synchronous_standby_names content
             return [x.strip().strip('"') for x in names_list.split(",")]
 
+    def send_heartbeat_query(self):
+        """
+        Sends a heartbeat query to the server with the already opened connection.
+
+        :returns tuple[bool, Exception|None]: A tuple where the first value is a boolean
+            indicating if the query executed successfully or not and the second is the
+            exception raised by ``psycopg2`` in case it did not succeed.
+        """
+        try:
+            with self._conn.cursor() as cursor:
+                cursor.execute(self.HEARTBEAT_QUERY)
+                _logger.debug("Sent heartbeat query to maintain the current connection")
+            return True, None
+        except psycopg2.Error as ex:
+            _logger.debug(
+                "Failed to execute heartbeat query on the current connection: %s"
+                % force_str(ex)
+            )
+            return False, ex
+
     @property
     def name_map(self):
         """
@@ -1776,3 +1879,108 @@ class StandbyPostgreSQLConnection(PostgreSQLConnection):
         return self._stop_backup(
             super(StandbyPostgreSQLConnection, self).stop_exclusive_backup
         )
+
+
+class PostgresKeepAlive:
+    """
+    Context manager to maintain a Postgres connection alive.
+
+    A child thread is spawned to execute heartbeat queries in the background
+    at a specified interval during its living context.
+
+    It does not open or close any connections on its own. Instead, it waits for the
+    specified connection to be opened on the main thread before start sending any query.
+
+    :cvar THREAD_NAME: The name identifying the keep-alive thread.
+    """
+
+    THREAD_NAME = "barman_keepalive_thread"
+
+    def __init__(self, postgres, interval, raise_exception=False):
+        """
+        Constructor.
+
+        :param barman.postgres.PostgreSQLConnection postgres: The
+            Postgres connection to keep alive.
+        :param int interval: An interval in seconds at which a
+            heartbeat query will be sent to keep the connection alive.
+            A value <= ``0`` won't start the keepalive.
+        :param bool raise_exception: A boolean indicating if an exception
+            should be raised in case the connection is lost. If ``True``, a
+            ``PostgresConnectionLost`` exception will be raised as soon as
+            it's noticed a connection failure. If ``False``, it will keep executing
+            normally until the context exits.
+        """
+        self.postgres = postgres
+        self.interval = interval
+        self.raise_exception = raise_exception
+        self._stop_thread = threading.Event()
+        self._thread = threading.Thread(
+            target=self._run_keep_alive,
+            name=self.THREAD_NAME,
+        )
+
+    def _prepare_signal_handler(self):
+        """
+        Set up a signal handler to raise an exception on the main thread when
+        the keep-alive thread wishes to interrupt it. This method listens for a
+        ``SIGUSR1`` signal and, when received, raises a ``PostgresConnectionLost``
+        exception.
+
+        .. note::
+            This code is, and only works if, executed while on the main thread. We
+            are not able to set a signal listener on a child thread, therefore this
+            method must be executed before the keep-alive thread starts.
+        """
+
+        def raise_exception(signum, frame):
+            raise PostgresConnectionLost("Connection to Postgres server was lost.")
+
+        signal.signal(signal.SIGUSR1, raise_exception)
+
+    def _raise_exception_on_main(self):
+        """
+        Trigger an exception on the main thread at whatever frame is being executed
+        at the moment. This is done by sending a ``SIGUSR1`` signal to the process,
+        which will be caught by the signal handler set previously in this class.
+
+        .. note::
+            This is an alternative way of interrupting the main thread's work, since
+            there is no direct way of killing or raising exceptions on the main thread
+            from a child thread in Python. A handler for this signal has been set
+            beforehand by the ``_prepare_signal_handler`` method in this class.
+        """
+        os.kill(os.getpid(), signal.SIGUSR1)
+
+    def _run_keep_alive(self):
+        """Runs the keepalive until a stop-thread event is set"""
+        while not self._stop_thread.is_set():
+            if not self.postgres.has_connection:
+                # Wait for the connection to be opened on the main thread
+                time.sleep(1)
+                continue
+
+            success, ex = self.postgres.send_heartbeat_query()
+
+            if not success and self.raise_exception:
+                # If one of the below exeptions was raised by psycopg2, it most likely
+                # means that the connection (and consequently, the session) was lost. In
+                # such cases, we can stop the keep-alive exection and raise the exception
+                if isinstance(ex, (psycopg2.InterfaceError, psycopg2.OperationalError)):
+                    self._stop_thread.set()
+                    self._raise_exception_on_main()
+
+            self._stop_thread.wait(self.interval)
+
+    def __enter__(self):
+        """Enters context. Starts the thread"""
+        if self.interval > 0:
+            if self.raise_exception:
+                self._prepare_signal_handler()
+            self._thread.start()
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        """Exits context. Makes sure the thread is terminated"""
+        if self.interval > 0:
+            self._stop_thread.set()
+            self._thread.join()
