@@ -132,7 +132,7 @@ def configure_logging(
     logging.root.addHandler(handler)
     if warn:
         # this will be always displayed because the default level is WARNING
-        _logger.warn(warn)
+        _logger.warning(warn)
     logging.root.setLevel(log_level)
 
 
@@ -514,6 +514,32 @@ def simplify_version(version_string):
     return ".".join(version[:-1])
 
 
+def get_major_version(text_string):
+    """
+    Finds a version number in a string and returns its major component.
+
+    :param text_string: The string containing the version number.
+    :return: The major version number as a string, or ``None`` if not found.
+    """
+    if text_string is None:
+        return None
+
+    # Step 1: Find the first number (integer or decimal) in the string.
+    # The pattern r'\d+\.?\d*' finds sequences of digits like "17" or "17.5".
+    match = re.search(r"\d+\.?\d*", text_string)
+
+    if not match:
+        return None  # No version number found in the string
+
+    # Step 2: Extract the matched version string (e.g., "17.5").
+    version_string = match.group(0)
+
+    # Step 3: Simplify it by taking the part before the first period.
+    major_version = version_string.split(".")[0]
+
+    return major_version
+
+
 def with_metaclass(meta, *bases):
     """
     Function from jinja2/_compat.py. License: BSD.
@@ -820,6 +846,61 @@ def check_aws_snapshot_lock_mode(value):
     return value
 
 
+def check_tag(value):
+    """
+    Parses and validates a tag string from the ``key,value`` format.
+
+    This function checks for character, length, and prefix constraints
+    as defined by AWS tag requirements.
+
+    :param value: The input string, e.g., ``owner,data-team``.
+    :return: A tuple of (key, value) if valid, or ``None`` if the input is empty.
+    :raises ValueError: If the format is wrong or constraints are not met.
+    """
+    if not value:
+        return None
+
+    # Split only on the first comma to allow commas in the value.
+    kv_lst = value.split(",", 1)
+    if len(kv_lst) != 2:
+        raise ValueError("Invalid tag format. Expected 'key,value'.")
+
+    # --- 1. Parse the Key and Value ---
+    key, val = kv_lst
+    key = key.strip()
+    val = val.strip()
+
+    # --- 2. Define Validation Rules ---
+    # Regex for allowed characters: letters, numbers, spaces, and _ . : / = + - @
+    # The hyphen is escaped (\-) to be treated as a literal.
+    allowed_chars_pattern = re.compile(r"^[a-zA-Z0-9\s_.:/=+\-@]*$")
+
+    # --- 3. Validate the Key ---
+    if not 1 <= len(key) <= 128:
+        raise ValueError("Tag key must be between 1 and 128 characters.")
+
+    if key.lower().startswith("aws:"):
+        raise ValueError("Tag key cannot start with the reserved prefix 'aws:'.")
+
+    if not allowed_chars_pattern.match(key):
+        raise ValueError(
+            "Tag key contains invalid characters. "
+            "Allowed are letters, numbers, spaces, and _.:/=+-@"
+        )
+
+    # --- 4. Validate the Value ---
+    if len(val) > 256:
+        raise ValueError("Tag value must be 256 characters or less.")
+
+    if not allowed_chars_pattern.match(val):
+        raise ValueError(
+            "Tag value contains invalid characters. "
+            "Allowed are letters, numbers, spaces, and _.:/=+-@"
+        )
+
+    return key, val
+
+
 def check_tli(value):
     """
     Check for a positive integer option, and also make "current" and "latest" acceptable values
@@ -1185,3 +1266,54 @@ def parse_target_tli(obj, target_tli, backup_info=None):
         else:
             raise ValueError("'%s' is not a valid timeline keyword" % target_tli)
     return parsed_target_tli
+
+
+@contextmanager
+def muted(logger):
+    """
+    Context manager to temporarily prevent a logger from printing any kind of
+    messages to the console.
+
+    This effectively mutes ``INFO``, ``WARNING``, and ``ERROR`` messages.
+    ``DEBUG`` messages can still logged in the log file.
+
+    This is useful to hide console outputs during internal operations where it's not
+    worth changing all methods involved to contain a ``verbose`` parameter or similar.
+
+    .. note::
+        As we are modifying an object that is used all throughout the code, it is worth
+        mentioning that it is not thread-safe. It should only be used in
+        single-threaded contexts or where the logger is not shared across threads.
+
+    :param logger: The logger to be muted
+    """
+    # Save original logging methods
+    original_info = logger.info
+    original_warning = logger.warning
+    original_error = logger.error
+
+    # Replace logging methods with no-op functions
+    logger.info = lambda *args, **kwargs: None
+    logger.warning = lambda *args, **kwargs: None
+    logger.error = lambda *args, **kwargs: None
+
+    try:
+        yield
+    finally:
+        # Restore original methods
+        logger.info = original_info
+        logger.warning = original_warning
+        logger.error = original_error
+
+
+def is_subdirectory(path1, path2):
+    """
+    Check if *path2* is a subdirectory of *path1*.
+
+    :param str path1: The parent directory path.
+    :param str path2: The child directory path.
+    :return bool: ``True`` if *path2* is a subdirectory of *path1, ``False`` otherwise
+    """
+    path1 = os.path.abspath(path1)
+    path2 = os.path.abspath(path2)
+    return os.path.commonpath([path1, path2]) == path1
